"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformColumnDefinitionsIntoAgGrid = transformColumnDefinitionsIntoAgGrid;
exports.translateAgGridFilter = translateAgGridFilter;
exports.translateColumnTypeIntoAgColumnType = translateColumnTypeIntoAgColumnType;
exports.translateColumnTypeIntoAgFilterType = translateColumnTypeIntoAgFilterType;
exports.translateAgFilterConditionIntoDV = translateAgFilterConditionIntoDV;

var _visualAnalyticsComponents = require("visual-analytics-components");

var _ag_interfaces = require("./ag_interfaces");

function transformColumnDefinitionsIntoAgGrid(columns) {
  return columns.filter( // ignore columns that are objects or arrays. They introduce systematic
  // complexities WRT filtering and sorting, and they will slow down the
  // app by possibly blowing up the DOM weight with large structures.
  function (c) {
    return c.type === _visualAnalyticsComponents.DatumType.String || c.type === _visualAnalyticsComponents.DatumType.Number || c.type === _visualAnalyticsComponents.DatumType.Date || c.type === _visualAnalyticsComponents.DatumType.Boolean;
  }).map(function (_ref) {
    var name = _ref.name,
        type = _ref.type;
    return {
      headerName: name,
      field: name,
      sortable: false,
      filter: translateColumnTypeIntoAgFilterType(type),
      type: translateColumnTypeIntoAgColumnType(type)
    };
  });
}

function getColumnNameSet(columns) {
  var columnNameMap = {};
  columns.forEach(function (c) {
    return columnNameMap[c.name] = true;
  });
  return columnNameMap;
}

function resolveFieldName(fieldName, knownColumnNames) {
  // HACK: Ag-Grid will append a suffix to the fieldName (e.g. _1) when we
  // switch between tables that share a field in common (e.g. 'weight' in
  // graph tables). We do this to map back to the original fieldName.
  //
  // Our strategy to find the original fieldname is to consider field names
  // that the ag-field starts with and then select the longest of those
  // candidates.
  //
  if (!knownColumnNames[fieldName]) {
    var longest = '';
    Object.keys(knownColumnNames).forEach(function (candidate) {
      if (fieldName.startsWith(candidate) && candidate.length > longest.length) {
        longest = candidate;
      }
    });
    return longest;
  } else {
    return fieldName;
  }
}

function translateAgGridFilter(filter, columns) {
  var result = {
    operation: _visualAnalyticsComponents.BooleanOperation.AND,
    clauses: []
  }; // Determine what column names exist in the source table.

  var knownColumnNames = getColumnNameSet(columns);
  Object.keys(filter).forEach(function (fieldName) {
    var fieldFilter = filter[fieldName];
    fieldName = resolveFieldName(fieldName, knownColumnNames);

    if (fieldFilter.operator) {
      var booleanFieldFilter = fieldFilter; // Compound Operator

      var cond1 = translateAgFilterItem(fieldName, booleanFieldFilter.condition1);
      var cond2 = translateAgFilterItem(fieldName, booleanFieldFilter.condition2);

      if (booleanFieldFilter.operator === 'OR') {
        result.clauses.push({
          operation: _visualAnalyticsComponents.BooleanOperation.OR,
          clauses: [cond1, cond2]
        });
      } else {
        result.clauses.push(cond1, cond2);
      }
    } else {
      var directFieldFilter = fieldFilter;
      result.clauses.push(translateAgFilterItem(fieldName, directFieldFilter));
    }
  });
  return result;
}

function translateAgFilterItem(field, filter) {
  return {
    field: field,
    value: filter.filter,
    operation: translateAgFilterConditionIntoDV(filter.type)
  };
}

function translateColumnTypeIntoAgColumnType(columnType) {
  switch (columnType) {
    case _visualAnalyticsComponents.DatumType.String:
      return undefined;

    case _visualAnalyticsComponents.DatumType.Number:
      return 'numericColumn';

    case _visualAnalyticsComponents.DatumType.Date:
      return 'dateColumn';

    default:
      return undefined;
  }
}

function translateColumnTypeIntoAgFilterType(columnType) {
  switch (columnType) {
    case _visualAnalyticsComponents.DatumType.String:
      return true;

    case _visualAnalyticsComponents.DatumType.Number:
      return 'agNumberColumnFilter';

    case _visualAnalyticsComponents.DatumType.Date:
      return 'agDateColumnFilter';

    default:
      return undefined;
  }
}

function translateAgFilterConditionIntoDV(filterType) {
  switch (filterType) {
    case _ag_interfaces.AGFilterCondition.Contains:
      return _visualAnalyticsComponents.FilterOperation.Contains;

    case _ag_interfaces.AGFilterCondition.NotContains:
      return _visualAnalyticsComponents.FilterOperation.NotContains;

    case _ag_interfaces.AGFilterCondition.StartsWith:
      return _visualAnalyticsComponents.FilterOperation.StartsWith;

    case _ag_interfaces.AGFilterCondition.EndsWith:
      return _visualAnalyticsComponents.FilterOperation.EndsWith;

    case _ag_interfaces.AGFilterCondition.Equals:
      return _visualAnalyticsComponents.FilterOperation.Equal;

    case _ag_interfaces.AGFilterCondition.NotEqual:
      return _visualAnalyticsComponents.FilterOperation.NotEqual;

    case _ag_interfaces.AGFilterCondition.GreaterThan:
      return _visualAnalyticsComponents.FilterOperation.GreaterThan;

    case _ag_interfaces.AGFilterCondition.GreaterThanOrEqual:
      return _visualAnalyticsComponents.FilterOperation.GreaterThanOrEqual;

    case _ag_interfaces.AGFilterCondition.LessThan:
      return _visualAnalyticsComponents.FilterOperation.LessThan;

    case _ag_interfaces.AGFilterCondition.LessThanOrEqual:
      return _visualAnalyticsComponents.FilterOperation.LessThanOrEqual;

    default:
      return undefined;
  }
}