/*!
 * Copyright (c) Microsoft. All rights reserved.
 * Licensed under the MIT license. See LICENSE file in the project.
 */
import * as React from 'react';
import { memo, useState, useCallback, useMemo, useEffect } from 'react';
import { filter } from 'rxjs/operators';
import styled from 'styled-components';
import { SliceType, MessageCreators, MessageTypes, } from 'visual-analytics-components';
import { AgFilterStore } from '../util/AgFilterStore';
import { Controls } from './Controls';
import { GlobalStyles } from './GlobalStyles';
import { Grid } from './Grid';
export const Datatable = memo(function Datatable({ manager, }) {
    const tableNames = useTableNames(manager);
    const store = useMemo(() => new AgFilterStore(), []);
    const [slice, setSlice] = useState(SliceType.FilteredIn);
    const [table, setTable] = useState(tableNames[0]);
    const filterLocation = useMemo(() => ({ table, key: 'datatable' }), [table]);
    const dataTable = useTable(manager, table);
    const dataSlice = useMemo(() => dataTable.slice(slice), [dataTable, slice]);
    const [dataVersion, setDataVersion] = useState(undefined);
    const [filterExpression, setFilterExpression] = useState(() => dataTable.slice(slice).filter.expressions['datatable']);
    const onFilterChanged = useCallback((expr, agExpr) => {
        if (expr) {
            store.setFilter(table, slice, agExpr);
            setFilterExpression(expr);
            manager.publish(MessageCreators.Filter.replace(filterLocation, expr, 'Apply Data Filter'));
        }
        else {
            store.clearFilter(table, slice);
            manager.publish(MessageCreators.Filter.clear(filterLocation, 'Clear Data Filter'));
        }
    }, [filterLocation, manager, slice, store, table]);
    const handleTableChanged = useCallback((tableName) => {
        console.log('set table', tableName);
        setTable(tableName);
    }, []);
    const handleSliceChanged = useCallback((slice) => setSlice(slice), []);
    const isDataChangeEvent = useCallback((evt) => {
        return (evt.type === MessageTypes.Data.Changed &&
            evt.payload.table === table &&
            evt.payload.slice === slice);
    }, [table, slice]);
    useEffect(() => {
        const subscription = manager.messages
            .pipe(filter(isDataChangeEvent))
            .subscribe((evt) => {
            setFilterExpression(dataTable.slice(slice).filter.expressions['datatable']);
            setDataVersion(evt);
        });
        return () => subscription.unsubscribe();
    }, [manager, isDataChangeEvent, dataTable, slice]);
    return (React.createElement(React.Fragment, null,
        React.createElement(GlobalStyles, null),
        React.createElement(Container, null,
            React.createElement(Controls, { tableNames: tableNames, table: table, slice: slice, onTableChanged: handleTableChanged, onSliceChanged: handleSliceChanged }),
            React.createElement(Grid, { filterExpression: filterExpression, agFilterExpression: store.getFilter(table, slice), dataVersion: dataVersion, columns: dataTable.columns, slice: dataSlice, onFilterChanged: onFilterChanged }))));
});
function useTableNames(manager) {
    return useMemo(() => manager.data.tables, [manager]);
}
function useTable(manager, name) {
    return useMemo(() => manager.data.table(name), [manager, name]);
}
const Container = styled.div `
	width: 100%;
	height: 100%;
	display: flex;
	flex-direction: column;
`;
