import { DatumType, BooleanOperation, FilterOperation, } from 'visual-analytics-components';
import { AGFilterCondition, } from './ag_interfaces';
export function transformColumnDefinitionsIntoAgGrid(columns) {
    return columns
        .filter(
    // ignore columns that are objects or arrays. They introduce systematic
    // complexities WRT filtering and sorting, and they will slow down the
    // app by possibly blowing up the DOM weight with large structures.
    c => c.type === DatumType.String ||
        c.type === DatumType.Number ||
        c.type === DatumType.Date ||
        c.type === DatumType.Boolean)
        .map(({ name, type }) => ({
        headerName: name,
        field: name,
        sortable: false,
        filter: translateColumnTypeIntoAgFilterType(type),
        type: translateColumnTypeIntoAgColumnType(type),
    }));
}
function getColumnNameSet(columns) {
    const columnNameMap = {};
    columns.forEach(c => (columnNameMap[c.name] = true));
    return columnNameMap;
}
function resolveFieldName(fieldName, knownColumnNames) {
    // HACK: Ag-Grid will append a suffix to the fieldName (e.g. _1) when we
    // switch between tables that share a field in common (e.g. 'weight' in
    // graph tables). We do this to map back to the original fieldName.
    //
    // Our strategy to find the original fieldname is to consider field names
    // that the ag-field starts with and then select the longest of those
    // candidates.
    //
    if (!knownColumnNames[fieldName]) {
        let longest = '';
        Object.keys(knownColumnNames).forEach(candidate => {
            if (fieldName.startsWith(candidate) && candidate.length > longest.length) {
                longest = candidate;
            }
        });
        return longest;
    }
    else {
        return fieldName;
    }
}
export function translateAgGridFilter(filter, columns) {
    const result = { operation: BooleanOperation.AND, clauses: [] };
    // Determine what column names exist in the source table.
    const knownColumnNames = getColumnNameSet(columns);
    Object.keys(filter).forEach(fieldName => {
        const fieldFilter = filter[fieldName];
        fieldName = resolveFieldName(fieldName, knownColumnNames);
        if (fieldFilter.operator) {
            const booleanFieldFilter = fieldFilter;
            // Compound Operator
            const cond1 = translateAgFilterItem(fieldName, booleanFieldFilter.condition1);
            const cond2 = translateAgFilterItem(fieldName, booleanFieldFilter.condition2);
            if (booleanFieldFilter.operator === 'OR') {
                result.clauses.push({
                    operation: BooleanOperation.OR,
                    clauses: [cond1, cond2],
                });
            }
            else {
                result.clauses.push(cond1, cond2);
            }
        }
        else {
            const directFieldFilter = fieldFilter;
            result.clauses.push(translateAgFilterItem(fieldName, directFieldFilter));
        }
    });
    return result;
}
function translateAgFilterItem(field, filter) {
    return {
        field,
        value: filter.filter,
        operation: translateAgFilterConditionIntoDV(filter.type),
    };
}
export function translateColumnTypeIntoAgColumnType(columnType) {
    switch (columnType) {
        case DatumType.String:
            return undefined;
        case DatumType.Number:
            return 'numericColumn';
        case DatumType.Date:
            return 'dateColumn';
        default:
            return undefined;
    }
}
export function translateColumnTypeIntoAgFilterType(columnType) {
    switch (columnType) {
        case DatumType.String:
            return true;
        case DatumType.Number:
            return 'agNumberColumnFilter';
        case DatumType.Date:
            return 'agDateColumnFilter';
        default:
            return undefined;
    }
}
export function translateAgFilterConditionIntoDV(filterType) {
    switch (filterType) {
        case AGFilterCondition.Contains:
            return FilterOperation.Contains;
        case AGFilterCondition.NotContains:
            return FilterOperation.NotContains;
        case AGFilterCondition.StartsWith:
            return FilterOperation.StartsWith;
        case AGFilterCondition.EndsWith:
            return FilterOperation.EndsWith;
        case AGFilterCondition.Equals:
            return FilterOperation.Equal;
        case AGFilterCondition.NotEqual:
            return FilterOperation.NotEqual;
        case AGFilterCondition.GreaterThan:
            return FilterOperation.GreaterThan;
        case AGFilterCondition.GreaterThanOrEqual:
            return FilterOperation.GreaterThanOrEqual;
        case AGFilterCondition.LessThan:
            return FilterOperation.LessThan;
        case AGFilterCondition.LessThanOrEqual:
            return FilterOperation.LessThanOrEqual;
        default:
            return undefined;
    }
}
